#
# Copyright (c) 2006 Mike L. Austin.  All rights reserved.
# see LICENCE.txt for details
#

Dir.chdir( "ruby-inertia" )

require 'opengl'
require 'glut'
require 'sdl'

require 'inertia'
require 'widgets/system/inspector'
require 'widgets/system/objectpalette'
require 'widgets/system/classeditor'
require 'widgets/clock'
require 'widgets/ball'

$screen_width, $screen_height = 1024, 768
#$screen_width, $screen_height = 800, 600

GL_LINE_SMOOTH_HINT = 0x0C52
GL_NICEST           = 0x1102

# ---------------------------------------------------------------------------- #

def initGL()
  puts "Setting up OpenGL..."
  SDL::init( SDL::INIT_VIDEO )
  SDL::setGLAttr( SDL::GL_DEPTH_SIZE, 32 )
  SDL::setGLAttr( SDL::GL_DOUBLEBUFFER, 1 )
  SDL::setVideoMode( $screen_width, $screen_height, 32, SDL::OPENGL )
  #SDL::setVideoMode( 0, 0, 32, SDL::OPENGL | SDL::FULLSCREEN )
  SDL::get_video_surface()
  GL::Enable( GL::DITHER )
  SDL::Key.enableKeyRepeat( 300, 30 )
  SDL::WM::setCaption( "Ruby-Inertia", "Ruby-Inertia" )

  GL::MatrixMode( GL::PROJECTION )
    GL::LoadIdentity()
    GLU::Ortho2D( 0, $screen_width, $screen_height, 0 )
  GL::MatrixMode( GL::MODELVIEW )
  GL::Viewport( 0, 0, $screen_width, $screen_height )

  GL::Enable( GL::BLEND );  GL::BlendFunc( GL::SRC_ALPHA, GL::ONE_MINUS_SRC_ALPHA )
  GL::Enable( GL::LINE_SMOOTH );  GL::Hint( GL_LINE_SMOOTH_HINT, GL_NICEST )  
  GL::Enable( GL::CULL_FACE );  GL::CullFace( GL::BACK )
end

initGL()

# ---------------------------------------------------------------------------- #

puts "Creating windows..."

class Inertia::Main

  include Inertia

  $systemFont = Font.new( "VeraNormal.ttf", 13 )

  $screen = Screen.new.with do
    self.extent = [$screen_width, $screen_height]
  end

  $login_window = Window.new.with do
    self.origin = [720, 10];  self.extent = [290, 170]
    self.title = "Log in to Database"

    add_child( Button.new.with do
      self.caption = "Log in"
      self.origin = [300 - 130, 180 - 80]
      self.text_color = [0.0, 0.5, 0.75]
      self.reshape = [:move, :move]
    end )

    add_child( Button.new.with do
      self.caption = "Cancel"
      self.origin = [20, 180 - 80]
      self.text_color = [0.0, 0.5, 0.75]
      self.reshape = [nil, :move]
    end )

    add_child( Label.new.with do
      self.text = "Username"
      self.origin = [20, 20]
    end )

    add_child( Input.new.with do
      self.origin = [90, 20];  self.extent = [180, 24]
      self.reshape = [:size, nil]
    end )

    add_child( Label.new.with do
      self.text = "Password"
      self.origin = [20, 50]
    end )

    add_child( Input.new.with do
      self.origin = [90, 50];  self.extent = [180, 24]
      self.reshape = [:size, nil]
      self.text = "abc123"
    end )
  end

  $screen.add_child( $login_window )
  $screen.add_child( ObjectPalette.new.with do
    self.origin = [720, 195]
  end )
  
  $screen.add_child( Scene.new.with do
    self.origin = [720, 435];  self.extent = [290, 290]
  end )
  
  $screen.add_child( Shelf.new )
  $screen.add_child( Ball.new )
end

# - main --------------------------------------------------------------------- #

event = SDL::Event.new
SDL::Event2.enable_unicode

$mouse_button = nil
$mouse_origin = Inertia::Point.new( 0, 0 )

puts "Entering main loop..."
#require 'profile'

include Inertia

loop do
  while event = SDL::Event2.poll
    case event
    when SDL::Event2::KeyDown
      print event.sym, " ", event.unicode, "\n"
      exit if event.unicode == SDL::Key::ESCAPE
      $screen.send_event( KeyEvent.new( :key_down, event.sym, event.unicode ) )
    when SDL::Event2::MouseButtonDown
      $mouse_button = event.button
      $screen.send_event( MouseEvent.new( :mouse_down, event.button, [event.x, event.y] ) )
    when SDL::Event2::MouseButtonUp
      $mouse_button = nil
      $screen.send_event( MouseEvent.new( :mouse_up, event.button, [event.x, event.y] ) )
    when SDL::Event2::MouseMotion
      $mouse_origin = Point.new( event.x, event.y )
      if $mouse_button
        $screen.send_event( MouseEvent.new( :mouse_drag, $mouse_button, [event.x, event.y] ) )
      else
        $screen.send_event( MouseEvent.new( :mouse_move, $mouse_button, [event.x, event.y] ) )
      end
    when SDL::Event2::Quit
      exit()
    end
  end
  
  GL::ClearColor( 0.0, 0.0, 0.0, 1.0 );
  GL::Clear( GL::COLOR_BUFFER_BIT | GL::DEPTH_BUFFER_BIT );
  GL::LoadIdentity();

  $screen.draw()

  if $screen.dragged_object
    GL::Translate( $mouse_origin.x - $screen.first_mouse.x,
                   $mouse_origin.y - $screen.first_mouse.y, 0 )
    $screen.grabbed_view.draw_shadow()
    GL::PushMatrix()  
      GL::Translate( -$screen.grabbed_view.screen_origin.x, -$screen.grabbed_view.screen_origin.y, 0 )
      $screen.grabbed_view.draw()
    GL::PopMatrix()
  end
  
  SDL::GLSwapBuffers()
end
