#
# Copyright (c) 2006 Mike L. Austin.  All rights reserved.
# see LICENCE.txt for details
#

require 'geometry'

module Inertia

class Event < Object

  @@types = :mouse_down, :mouse_up, :mouse_move, :mouse_drag,
            :mouse_enter, :mouse_exit, :mouse_drop, :view_size,
            :key_down, :key_up

  attr :type, true

end

class MouseEvent < Event

  attr :button
  attr :origin, true
  attr :object, true

  def initialize( type, button, origin, object = nil )
    @type = type
    @button = button
    @origin = origin.to_point
    @object = object
    freeze
  end

  def origin=( value )
    @origin = value.to_point
  end
end

class KeyEvent < Event

  attr :keysym, true
  attr :value, true

  def initialize( type, keysym, value )
    @type = type
    @keysym = keysym
    @value = value
  end

end

class ShapeEvent < Event

  attr :origin, true

  def initialize( type, delta )
    @type = type
    @origin = Point.new( 0, 0 )
    self.origin = delta
  end

  def origin=( value )
    @origin = value.to_point
  end
end

class Action

attr :target
  attr :message
  attr :args
  
  def initialize( target, message, args )
    @target, @message, @args = target, message, args
  end
  
  def call()
    @target.send( @message, *@args )
  end

end

end
