#
# Copyright (c) 2006 Mike L. Austin.  All rights reserved.
# see LICENCE.txt for details
#

module Inertia

class Font < Object

  SDL::TTF::init()

  attr :font
  attr :sizes
  attr :tex_coords
  attr :surface

  # - Font initialization ---------------------------------------------------- #

  def initialize( filename, size )
    @font = SDL::TTF::open( filename, size )
    @surface = SDL::Surface.new( SDL::SWSURFACE, 256, 256, 24, 0xFF, 0xFF, 0xFF, 0xFF )

    x, y = 0, 0
    height = font.height
    size = font.height
    @sizes = (32..128).map do |char|
      font.draw_blended_utf8( @surface, char.chr, x, y, 255, 255, 255 )
      x += height
      if x % 10 == 0
        y += height
        x = 0
      end
      font.text_size( char.chr )
    end

    @sizes["j"[0]-32][0] -= 1
    @sizes["k"[0]-32][0] -= 1
    
    @tex_coords = (32..128).map do |char|
      [((char - 32) % 10 * size) / 256.0,
       ((char - 32) % 10 * size + size) / 256.0,
       ((char - 32) / 10 * size) / 256.0,
       ((char - 32) / 10 * size + size) / 256.0]
    end

    pixels = surface.pixels
    s = String.new

    for i in 0..262144
      s << pixels[i * 3].to_i
    end

    GL::BindTexture( GL::TEXTURE_2D, 1 )
    GL::TexEnv( GL::TEXTURE_ENV, GL::TEXTURE_ENV_MODE, GL::MODULATE )
    GL::TexParameter( GL::TEXTURE_2D, GL::TEXTURE_MIN_FILTER, GL::LINEAR )
    GL::TexParameter( GL::TEXTURE_2D, GL::TEXTURE_MAG_FILTER, GL::LINEAR )
    GL::PixelStorei( GL::UNPACK_ALIGNMENT, 1 ) 
    GL::TexImage2D( GL::TEXTURE_2D, 0, GL::ALPHA, 256, 256, 0, GL::ALPHA, GL::UNSIGNED_BYTE, s )

    (32..128).each do | char |
      GL::NewList( char, GL::COMPILE )
        GL::Begin( GL::QUADS )
            offset = char - 32
            GL::TexCoord2f( @tex_coords[offset][0], @tex_coords[offset][2] );  GL::Vertex(    0,    0 )
            GL::TexCoord2f( @tex_coords[offset][0], @tex_coords[offset][3] );  GL::Vertex(    0, size )
            GL::TexCoord2f( @tex_coords[offset][1], @tex_coords[offset][3] );  GL::Vertex( size, size )
            GL::TexCoord2f( @tex_coords[offset][1], @tex_coords[offset][2] );  GL::Vertex( size,    0 )
        GL::End()
        GL::Translate( @sizes.at(offset).at(0), 0.0, 0.0 )
      GL::EndList()
    end

  end

  # - Font attributes -------------------------------------------------------- #

  def height()
    return font.height
  end

  def line_skip()
    return font.line_skip
  end
  
  def text_length( string )
    font.text_size( string )[0]
  end

  def index_for_width( string, width )
    w, i, x = 0, 0, 0
    string.each_byte do | char |
      x = @font.text_size( char.chr )[0]
      return i if w + (x / 2.0) > width 
      w += x
      i += 1
    end
    return i
  end

  def index_for_height( string_array, height )
    h, i, y = 0, 0, 0
    string_array.each do | line |
      y = font.line_skip 
      return i if h + y > height 
      h += y
      i += 1
    end
    return i
  end

  def height_for_index( string_array, index )
    h = 0
    skip = font.line_skip
    string_array.each_with_index do | line, i |
      return h if i > index - 1 
      h += skip
    end
    return h
  end
  
  # - Font drawing ----------------------------------------------------------- #

  def draw_string( value )
    string = value.to_s
    GL::Enable( GL::TEXTURE_2D )
    GL::PushMatrix()
      GL::CallLists( string )
    GL::PopMatrix()
    GL::Disable( GL::TEXTURE_2D )
  end

  def draw_strings( string_array, max_height )
    #count = height / font.line_skip
    skip = font.line_skip

    GL::Enable( GL::TEXTURE_2D )
    GL::PushMatrix()
      y = 0
      string_array.each_with_index do | string, i |
        break if y + skip >= max_height
        GL::PushMatrix()
          GL::CallLists( string )
        GL::PopMatrix()
        GL::Translate( 0.0, skip, 0.0 )
        y += skip
      end
    GL::PopMatrix()
    GL::Disable( GL::TEXTURE_2D )
  end

end # Font

end

