#
# Copyright (c) 2006 Mike L. Austin.  All rights reserved.
# see LICENCE.txt for details
#

require 'view'
require 'events'

module Inertia

class Screen < View

  @source = __FILE__

  attr :grabbed_view, true
  attr :hovered_view
  attr :dragged_object, true
  attr :first_mouse
  attr :meta_object

  # -------------------------------------------------------------------------- #
  class General; end
  # -------------------------------------------------------------------------- #

  def initialize()
    super
    screen = self
    @hovered_view = self
    @meta_object = nil
    @meta_mode = nil
    @meta_menu = Menu.new.with do
      add_item( Button.new.with do
        self.caption = "Inspect";  self.height = 19
        self.action = Action.new( screen, :inspect_view, [] )
      end )
      add_item( Button.new.with do
        self.caption = "Edit Class";  self.height = 19
        self.action = Action.new( screen, :edit_class, [] )
      end )
      add_item( Button.new.with do
        self.caption = "Subclass";  self.height = 19
        end )
      add_item( Button.new.with do
        self.caption = "Delete";  self.height = 19
        self.action = Action.new( screen, :delete_view, [] )
      end )
    end
    
    @root_menu = Menu.new.with do
      add_item( Button.new.with do
        self.caption = "New Workspace";  self.height = 19
      end )
      add_item( Button.new.with do
        self.caption = "Object Inspector";  self.height = 19
      end )
      add_item( Button.new.with do
        self.caption = "Class File Editor";  self.height = 19
      end )
      add_item( Separator.new )
      add_item( Button.new.with do
        self.caption = "New Shelf";  self.height = 19
      end )
    end
  end

  def maximize_child( child )
    min_x = @subviews.inject( 0 - 2 ) do | value, view |
      [value, view.left + view.width >= child.left ? 0 - 2 : view.left + view.width].max()
    end
    min_y = @subviews.inject( 0 - 2 ) do | value, view |
      [value, view.top + view.height >= child.top ? 0 - 2 : view.top + view.height].max()
    end
    max_x = @subviews.inject( width + 2 ) do | value, view |
      [value, view.left >= child.left + child.width ? view.left : width + 2].min()
    end
    max_y = @subviews.inject( height + 2 + 20 ) do | value, view |
      [value, view.top >= child.top + child.height ? view.top : height + 2 + 20].min()
    end

    child.origin = [min_x + 14, min_y + 34]
    child.extent = [max_x - child.left - 14, max_y - child.top - 34]
  end
  
  def drag_object( object )
    @dragged_object = object
  end

  def inspect_view()
    $screen.add_child( Inspector.new.with do
      self.object = $screen.meta_object
      self.origin = [$mouse_origin.x - width / 2, $mouse_origin.y - 10]
    end )
  end

  def edit_class()
    $screen.add_child( ClassEditor.new.with do
      self.origin = [$mouse_origin.x - width / 2, $mouse_origin.y - 10]
      open_file( $screen.meta_object.class.source )
    end )
  end

  def delete_view()
    $screen.meta_object.parent.remove_child( $screen.meta_object ) if $screen.meta_object.parent
  end

  def active_window=( window )
    @active_window.focus_lost() if @active_window
    @active_window = window
  end

  def focused_view=( view )
    @focused_view.focus_lost() if @focused_view
    @focused_view = view
  end
  
  # -------------------------------------------------------------------------- #
  class Events; end
  # -------------------------------------------------------------------------- #

  alias :view_send_event :send_event
  
  def handle_meta_op( event, mod_state )
      if event.type == :mouse_down
        view = view_at( event.origin )

        if SDL::Key::mod_state & SDL::Key::MOD_SHIFT != 0
           @grabbed_view = view.class.new
           @grabbed_view.origin = view.screen_origin.clone
           @grabbed_view.extent = view.extent.clone
        else
          @grabbed_view = view
        end

        if event.origin.x > view.screen_origin.x + view.width - 10 and
           event.origin.y > view.screen_origin.y + view.height - 10
          @first_mouse = (event.origin - @grabbed_view.screen_origin)
          @meta_mode = :size
        else
          @meta_mode = :move
        end

        @meta_object = view

        if event.button == SDL::Mouse::BUTTON_RIGHT
          @meta_menu.origin = event.origin
          $screen.remove_child( @meta_menu )
          $screen.add_child( @meta_menu )
        end
      elsif event.type == :mouse_drag and event.button == SDL::Mouse::BUTTON_LEFT
        if @meta_mode == :move and @grabbed_view != self
          if !@dragged_object
            @dragged_object = @grabbed_view
            @first_mouse = event.origin - @grabbed_view.screen_origin
            @grabbed_view.parent.remove_child( @grabbed_view ) if @grabbed_view.parent
          end
          @grabbed_view.origin = event.origin - @first_mouse
        elsif @meta_mode == :size
          @grabbed_view.extent = (event.origin - @grabbed_view.screen_origin)
        end
      elsif event.type == :mouse_up
        if @meta_mode == :move
          if @dragged_object
            view = $screen.view_at( event.origin )
            view.add_child( @grabbed_view )
            @grabbed_view.origin = event.origin - view.screen_origin - @first_mouse 
            @grabbed_view.origin = ((@grabbed_view.origin + Point.new(5, 5)) / 10) * 10
            @dragged_object = nil
          end
        elsif @meta_mode == :size
          @grabbed_view.extent = ((@grabbed_view.extent + Point.new(5, 5)) / 10) * 10
        end
        @grabbed_view = nil
      end
      return @grabbed_view
  end
  
  def send_event( event )
    # If Alt is pressed, move the view (note: move code to other class?)
    SDL::Key::scan
    if SDL::Key::mod_state & SDL::Key::MOD_ALT != 0
      return handle_meta_op( event, SDL::Key::mod_state )
    end

    # If we are in a drag & drop state, handle mouse_drag and mouse_up
    if @dragged_object
      if event.type == :mouse_drag
        super.send_event( MouseEvent.new( :drag_accept, event.button, event.origin, @dragged_object ) )
        return @grabbed_view
      elsif event.type == :mouse_up
        @grabbed_view = nil
        dropped_view = super.send_event( MouseEvent.new( :drag_drop, event.button, event.origin, @dragged_object ) )
        @dragged_object = nil
        return dropped_view
      end
    end

    # If mouse button change, set the grabbed view
    if event.type == :mouse_down
      @grabbed_view = super
      @first_mouse = event.origin - @grabbed_view.screen_origin
      return @grabbed_view
    elsif event.type == :mouse_up
      @grabbed_view = nil
    end

    # If mouse is grabbed, send event to it
    if @grabbed_view
      grabbed_event = MouseEvent.new( event.type, event.button, event.origin - @grabbed_view.screen_origin )
      if @grabbed_view == self
        return @grabbed_view.view_send_event( grabbed_event )
      else
        return @grabbed_view.send_event( grabbed_event )
      end
    end

    if event.type == :key_down and @focused_view
      return @focused_view.send_event( event )
    end

    view = super

    # Send mouse_enter and mouse_exit events
    if event.type == :mouse_move and view != hovered_view
      @hovered_view.send_event( MouseEvent.new( :mouse_exit, event.button,
                                event.origin - @hovered_view.screen_origin ) )
      @hovered_view = view
      @hovered_view.send_event( MouseEvent.new( :mouse_enter, event.button, event.origin ) )
    end

    return view
  end

  def mouse_down( button, origin )
    if button == SDL::Mouse::BUTTON_RIGHT
      @root_menu.origin = origin
      remove_child( @root_menu )
      add_child( @root_menu )
    end
  end

  # -------------------------------------------------------------------------- #
  class Drawing; end
  # -------------------------------------------------------------------------- #

  def draw_content()
    GL::Color( 1.0, 1.0, 1.0 )
    GL::Begin( GL::QUADS )
      GL::Vertex(     0,      0 )
      GL::Vertex(     0, height )
      GL::Vertex( width, height )
      GL::Vertex( width,      0 )
    GL::End()
  end
  
  def draw_border()
  end

end # Screen

end
