#
# Copyright (c) 2006 Mike L. Austin.  All rights reserved.
# see LICENCE.txt for details
#

module Inertia

class UIStyle

  def UIStyle.draw_content( view, graphics )
    graphics.solid_rect( Point.ZERO, view.extent, [view.back_color] ) if view.back_color
  end

  def UIStyle.draw_border( view, graphics )
    graphics.line_rect( Point.ZERO, view.extent, [view.border_color] ) if view.border_color
  end
  
end

module UIStyle::Intertia

  class Button < UIStyle

    @@text_color = [0.1, 0.1, 0.1]
    @@normal = [ [1.0, 1.0, 1.0], [0.9, 0.9, 0.9] ]
    @@hilite = [ [0.8, 0.9, 1.0], [0.9, 0.95, 1.0] ]
  
    def Button.draw_content( button, graphics )
      graphics.solid_rect( Point.ZERO, button.extent, button.mouse_down? ? @@hilite : @@normal )

      GL::Color( @@text_color ); 
      GL::Translate( button.width / 2 - $systemFont.text_length( button.caption ) / 2,
                     button.height / 2 - $systemFont.font.height / 2, 0.0 )
      $systemFont.draw_string( button.caption )
    end
  
    def Button.draw_border( button, graphics )
      graphics.line_width = 1.0      
      graphics.line_rect( Point.ZERO, button.extent, [0.75, 0.75, 0.75] )
    end

  end

  class HoverButton < Button

    def HoverButton.draw_content( button, graphics )
      return super if button.mouse_hover?

      GL::Color( @@text_color ); 
      graphics.translate( button.width / 2 - $systemFont.text_length( button.caption ) / 2,
                          button.height / 2 - $systemFont.font.height / 2, 0.0 )
      $systemFont.draw_string( button.caption )
    end

    def HoverButton.draw_border( button, graphics )
      super if button.mouse_hover?
    end

  end
  
  class Input < UIStyle
  
    def Input.draw_content( view, graphics )
      super
      GL::Color( view.text_color ); 
      GL::Translate( 5.0, view.height / 2 - $systemFont.font.height / 2, 0.0 )
      $systemFont.draw_string( view.text )
    end
    
  end

end

module UIStyle::Menu

  class Button < UIStyle::Intertia::Button

    def Button.draw_content( button, graphics )
      if button.mouse_hover?
        graphics.solid_rect( Point.ZERO, button.extent, button.mouse_down? ? @@hilite : @@normal )
      end

      GL::Color( @@text_color ); 
      GL::Translate( 20, button.height / 2 - $systemFont.font.height / 2, 0 )
      $systemFont.draw_string( button.caption )
    end

    def Button.draw_border( button, graphics )
      if button.mouse_hover?
        #graphics.line_width = 1.0
        #graphics.line_rect( Point.ZERO, button.extent, [0.75, 0.75, 0.75] )
      end
    end
    
  end

end

end
