#
# Copyright (c) 2006 Mike L. Austin.  All rights reserved.
# see LICENCE.txt for details
#

#
# Copyright (c) 2006 Mike L. Austin.  All rights reserved.
# see LICENCE.txt for details
#

require 'view'

module Inertia

class Caret

  def initialize( editor )
    @position = Point.new
    @editor = editor
  end

  def x()
    @position.x
  end

  def y()
    @position.y
  end
  
  def x=( x )
    line = @editor.lines[@position.y]
    if x < 0
      self.y -= 1
      @position.x = @editor.lines[@position.y].length
    elsif x > line.length
      self.y += 1
      @position.x = x - line.length - 1
    else
      @position.x = x
    end
  end

  def y=( y )
    if y >= 0 and y < @editor.lines.length
      @position.y = y
    end

    if @position.y < @editor.text_top 
      @editor.text_top -= 1
    end

    if (@position.y - @editor.text_top) * $systemFont.line_skip > @editor.height - $systemFont.line_skip - 10
      #@editor.text_top += y - 23 if @editor.text_top < @editor.lines.length
      @editor.text_top = @position.y - 23 if @editor.text_top < @editor.lines.length
    end
  end

end

# ---------------------------------------------------------------------------- #

class Editor < View

  @source = __FILE__
  @fields = [:text_color, :back_color]

  attr :lines
  attr :text_color, true
  attr :back_color, true
  attr :caret
  attr :text_top, true

  # -------------------------------------------------------------------------- #
  
  def initialize()
    super
    @lines = ["Hello", "World"]
    @lines = IO.readlines( "Widgets/Button.rb" )
    @lines.each do | line | line.chomp! end
    @text_color = [0.1, 0.1, 0.1]
    @back_color = [1.0, 1.0, 1.0]
    @caret = Caret.new( self ) #Point.new
    @text_top = 0
  end

  def load_from_file( filename )
    begin
      @lines = IO.readlines( filename )
      @lines.each do | line | line.chomp! end
    rescue
      puts "No such file"
    end
  end  
  
  def save_to_file( filename )
    file = File.open( filename, File::WRONLY | File::TRUNC )
    lines.each do | line | file << line << "\r\n" end
    file.close()
  end

  # -------------------------------------------------------------------------- #

  def left_arrow()
    @caret.x -= 1
  end

  def right_arrow()
    @caret.x += 1
  end
  
  def backspace()
    line = @lines[@caret.y]
    if @caret.x > 0
      @caret.x -= 1
      line.slice!( @caret.x )
    end
  end

  def delete()
    line = @lines[@caret.y]
    if @caret.x < line.length
      line.slice!( @caret.x )
    else
      if @caret.y < @lines.length - 1
        line[@caret.x, 0] = @lines[@caret.y + 1]
        @lines.slice!( @caret.y + 1 )
      end
    end
  end

  def down_arrow()
    w = $systemFont.text_length( @lines[@caret.y][0, @caret.x] )
    @caret.y += 1
    @caret.x = $systemFont.index_for_width( @lines[@caret.y], w )
  end

  def up_arrow()
    w = $systemFont.text_length( @lines[@caret.y][0, @caret.x] )
    @caret.y -= 1
    @caret.x = $systemFont.index_for_width( @lines[@caret.y], w )
  end
  
  def page_down()
    @caret.y += 23
  end

  def enter_key()
    line = @lines[@caret.y]
    @lines[@caret.y + 1, 0] = @lines[@caret.y][@caret.x, line.length]
    @lines[@caret.y][@caret.x, line.length] = ""
    @caret.y += 1
    @caret.x = 0
  end
  
  def insert( char )
    line = @lines[@caret.y]
    x = [@caret.x, line.length].min
    line[x, 0] = char
    @caret.x += 1
  end
  
  def key_down( keysym, value )
    line = @lines[@caret.y]

    case keysym
      when   8 then backspace()
      when  13 then enter_key()
      when 127 then delete()
      when 276 then left_arrow()
      when 275 then right_arrow()
      when 274 then down_arrow()
      when 273 then up_arrow()
      when 281 then page_down()
      else insert( value.chr ) if value > 31 and value < 127
    end
  end
  
  def mouse_down( button, origin )
    #@caret.y = (origin.y - 5) / $systemFont.line_skip
    @caret.y = $systemFont.index_for_height( @lines, origin.y - 5 ) + @text_top
    @caret.x = $systemFont.index_for_width( @lines[@caret.y], origin.x - 10 )
  end
  
  def draw_content()
    # Draw background
    GL::Color( back_color )
    GL::Begin( GL::QUADS )
      GL::Vertex(     0,      0 )
      GL::Vertex(     0, height )
      GL::Vertex( width, height )
      GL::Vertex( width,    0.0 )
    GL::End()

    # Draw text
    GL::Translate( 10, 5, 0 )
    GL::Color( text_color )
    #$systemFont.draw_strings( lines, height - 5 )
    $systemFont.draw_strings( lines[@text_top, lines.size], height - 5 )

    # Draw caret
    GL::LineWidth( 2.0 )
    if back_color.inject( 0 ) do | a, i | a + i end / 2 > 0.5
      GL::Color( 0.1, 0.1, 0.1 )
    else
      GL::Color( 1.0, 1.0, 1.0 )
    end
    GL::Begin( GL::LINES )
      line = @lines[@caret.y]
      skip = $systemFont.line_skip
      x = $systemFont.text_length( line[0, @caret.x] )
      #y = @caret.y * $systemFont.line_skip
      y = $systemFont.height_for_index( lines, @caret.y )
      GL::Vertex( x, y - (@text_top * skip) )
      GL::Vertex( x, y - (@text_top * skip) + skip )
    GL::End()
  end
  
end

class CaretModel < Model

  attr :position

end

class EditorModel < Model

  attr :lines

end

end
