#
# Copyright (c) 2006 Mike L. Austin.  All rights reserved.
# see LICENCE.txt for details
#

require 'view'

module Inertia

class Input < View

  @source = __FILE__

  attr :text_color
  attr :text, true
  attr :position
  attr :enter_action, true

  def initialize()
    super
    self.extent = [100, 25]
    @text_color = [0.1, 0.1, 0.1]
    @back_color = [1.0, 1.0, 1.0]
    @text = "someone@somewhere.com"
    @position = Point.new
  end

  def backspace()
    if @position.x > 0
      @position.x -= 1
      text.slice!( @position.x )
    end
  end

  def delete()
    if @position.x < text.length
      text.slice!( @position.x )
    end
  end

  def insert( char )
    @text[@position.x, 0] = char
    @position.x += 1 if @position.x < text.length
  end

  def left_arrow()
    @position.x -= 1 if @position.x > 0
  end

  def right_arrow()
    @position.x += 1 if @position.x < text.length
  end

  def enter_key()
    enter_action.call() if enter_action
  end
  
  def key_down( keysym, value )
    case keysym
      when   8 then backspace()
      when 127 then delete()
      when 276 then left_arrow()
      when 275 then right_arrow()
      when  13 then enter_key()
      else insert( value.chr ) if value > 31 and value < 127
    end
  end

  def mouse_down( button, origin )
    @position.x = $systemFont.index_for_width( @text, origin.x - 5 )
  end

  def draw_content()
    # Draw caret
    GL::LineWidth( 1.0 )
    GL::Color( 0.2, 0.2, 0.2 )

    GL::Begin( GL::LINES )
      x = $systemFont.text_length( text[0, @position.x] ) + 0.5
      GL::Vertex( x, 0 + 0.5)
      GL::Vertex( x, 0 + $systemFont.line_skip + 0.0 )
    GL::End()
  end

  def draw_border()
    #if $screen.active_window and $screen.active_window.focused_view == self 
    if @parent and @parent.focused_view == self 
      line_width = 1.0
      GL::LineWidth( 2.0 )
    else
      line_width = 0.5
      GL::LineWidth( 1.0 )
    end

    GL::Enable( GL::LINE_SMOOTH )
    GL::Color( 0.75, 0.75, 0.75, 1.0 ); 

    #if mouse_hover?
      GL::Begin( GL::LINE_LOOP )
        GL::Vertex(     0 - line_width,      0 - line_width )
        GL::Vertex(     0 - line_width, height + line_width )
        GL::Vertex( width + line_width, height + line_width )
        GL::Vertex( width + line_width,      0 - line_width )
      GL::End()
    #else
    #  GL::Begin( GL::LINE_STRIP )
    #    GL::Vertex(     0 - line_width / 2.0, height + line_width / 2.0 )
    #    GL::Vertex( width + line_width / 2.0, height + line_width / 2.0 )
    #  GL::End()
    #end
  end

end

end
