#
# Copyright (c) 2006 Mike L. Austin.  All rights reserved.
# see LICENCE.txt for details
#

require 'view'

module Inertia

class Separator < View

  def initialize()
    super
    self.height = 9
  end

  def draw_content()
    GL::LineWidth( 1.0 )
    GL::Color( 0.75, 0.75, 0.75 )
    GL::Begin( GL::LINES )
      GL::Vertex( 5, height / 2 + 0.5 )
      GL::Vertex( width - 5, height / 2 + 0.5 )
    GL::End()
  end

  def draw_border()
  end

end

class Menu < View
  
  @source = __FILE__

  def initialize()
    super
    @top = 11
    @text_color = [0.1, 0.1, 0.1]
    @back_color = [0.9375, 0.9375, 0.9375] 
    self.width = 20
  end
  
  def add_item( view )
    view.top = @top
    
    class_name = view.class.to_s.split("::").last
    view.uistyle = if UIStyle::Menu.constants.include? class_name
      UIStyle::Menu.const_get( class_name ) else UIStyle end
    
    add_child( view )
    @top += view.height
    self.width = [width, view.width].max()
    self.height = @top + 4
    
    @subviews.each do | child |
      child.width = width
    end
  end

  def mouse_down( button, origin )
    @first_mouse = origin
    @sticky = true
  end

  def mouse_drag( origin )
    self.origin += origin - @first_mouse
  end
  
  def send_event( event )
    @parent.remove_child( self ) if event.type == :mouse_up and !@sticky
    super
  end
  
  def draw_content()
    draw_shadow()

    # Draw background
    GL::Begin( GL::QUADS )
      GL::Color( 0.9375, 0.9375, 0.9375, 0.95 )
        GL::Vertex(     0, 100 )
        GL::Vertex( width, 100 )
      GL::Color( 1.0, 1.0, 1.0, 0.95 )
        GL::Vertex( width,      0 )
        GL::Vertex(     0,      0 )
      GL::Color( 0.9375, 0.9375, 0.9375, 0.95 )
        GL::Vertex(     0,    100 )
        GL::Vertex(     0, height )
        GL::Vertex( width, height )
        GL::Vertex( width,    100 )

      GL::Color( 0.75, 0.75, 0.75, 1.0 )
        GL::Vertex(     0,      0 )
        GL::Vertex(     0, 7 )
        GL::Vertex( width, 7 )
        GL::Vertex( width, 0 )
    GL::End()
  end
  
  def draw_border()
    half_width = 1
    GL::LineWidth( 2.0 )
    GL::Color( 0.75, 0.75, 0.75, 1.0 )
    GL::Begin( GL::LINE_LOOP )
      GL::Vertex(     0 - half_width,      0 - half_width )
      GL::Vertex(     0 - half_width, height + half_width )
      GL::Vertex( width + half_width, height + half_width )
      GL::Vertex( width + half_width,      0 - half_width )
    GL::End()
  end
    
end # Menu

end
