#
# Copyright (c) 2006 Mike L. Austin.  All rights reserved.
# see LICENCE.txt for details
#

require 'view'

module Inertia

class Scene < View

  @source = __FILE__
  @fields = [:angle, :fov]

  attr :angle
  attr :fov
  attr :model, true

  def angle=( point ) model.angle = point.to_point end
  def angle() return model.angle end

  def fov=( fov ) model.fov = fov; compile_dlist() end
  def fov() model.fov end
  
  def draw()
    GL::PushMatrix()
      GL::CallList( 1 )
        GL::Viewport( screen_origin.x, $screen.height - screen_origin.y - height, width, height )

        GL::LoadIdentity()
        GL::PushMatrix()
          draw_content()
        GL::PopMatrix()
      GL::CallList( 2 )
    GL::PopMatrix()
    GL::PushMatrix()
      GL::Translate( @origin.x, @origin.y, 0 )
      draw_children()
      draw_border()
    GL::PopMatrix()
  end

  def mouse_down( button, origin )
    @first_mouse = origin
  end
  
  def mouse_drag( origin )
    self.angle = angle + origin - @first_mouse
    @first_mouse = origin
  end
  
  def draw_content()
    GL::Color( 0.2, 0.6, 1.0 )
    GL::Translate( 0, 0, -4 )
    GL::Rotate( model.angle.y, 1, 0, 0 )
    GL::Rotate( model.angle.x, 0, 1, 0 )
    GLUT::SolidTeapot( 1 )
  end

  def initialize()
    super
    @model = SceneModel.new
    @first_mouse = nil
    self.extent = [300, 300]

    compile_dlist()    
  end
  
  def compile_dlist()
    GL::NewList( 1, GL::COMPILE )
      GL::PushAttrib( GL::TRANSFORM_BIT | GL::VIEWPORT_BIT )
        GL::MatrixMode( GL::PROJECTION )
          GL::PushMatrix()
          GL::LoadIdentity()
          GLU::Perspective( model.fov, width.to_f / height.to_f, 1, 10 )
        GL::MatrixMode( GL::MODELVIEW )

        GL::Enable( GL::DEPTH_TEST )
        GL::Enable( GL::LIGHTING );  GL::Enable( GL::LIGHT0 )
        GL::Disable( GL::CULL_FACE )
    GL::EndList()

    GL::NewList( 2, GL::COMPILE )
        GL::Enable( GL::CULL_FACE )
        GL::Disable( GL::LIGHTING )
        GL::Disable( GL::DEPTH_TEST )
      
        GL::MatrixMode( GL::PROJECTION )
          GL::PopMatrix()
        GL::MatrixMode( GL::MODELVIEW )
      GL::PopAttrib()
    GL::EndList()
  end
  
end

class SceneModel

  attr :listeners
  attr :angle
  attr :fov
  
  def initialize()
    @listeners = []
    @angle = Point.new
    @fov = 60
  end
  
  def angle=( point )
    @angle = point.to_point
    broadcast( SceneEvent.new( :value_changed, @angle ) )
  end

  def fov=( fov )
    @fov = fov.to_f
    broadcast( SceneEvent.new( :value_changed, @fov ) )
  end
  
  def add_listener( object )
    @listeners.push( object )
  end
  
  def broadcast( event )
    @listeners.each do | listener |
      listener.send_event( event )
    end
  end

end

class SceneEvent < Event

  def initialize( type, value )
    @type = type
    @value = value
  end

end

end
