#
# Copyright (c) 2006 Mike L. Austin.  All rights reserved.
# see LICENCE.txt for details
#

require 'view'

module Inertia

class ObjectModel

  class Items
    include Enumerable

    attr :items, true

    def initialize( parent )
      @parent = parent
      @items = @parent.object.class.fields.map do | var |
        [var.to_s, @parent.object.send( var ).inspect]
      end
    end
    
    def each( &block )
      items.each( &block )
    end

    def size()
      items.size
    end
    
    def []( item, column )
      @items[item][column].clone
    end
    
    def []=( item, index, value )
      begin
        @parent.object.send( "#{items[item][0]}=", value )
      rescue
        puts 'bad'
      end
    end
  end

  @@columns = [['Name', 120], ['Value', 200]]

  attr :object, true
  attr :columns
  attr :items, true

  def initialize()
    @object = $screen
    self.items = Items.new( self )
  end
  
  def columns()
    @@columns
  end
  
  def object=( object )
    @object = object
    @object.model.add_listener( self ) if @object.respond_to? :model
    @items = Items.new( self )
  end

  def send_event( event )
    if event.class == SceneEvent
      @items = Items.new( self )
    else
      super
    end
  end

end

class Table < View

@source = __FILE__

  attr :model, true
  attr :selected
  
  def initialize()
    super
    @input = Input.new
    @input.enter_action = Action.new( self, :change_value, [] )
  end
  
  def change_value()
    begin
      result = eval( @input.text )
      model.items[@selected, 1] = result
    rescue SyntaxError => e
      @input.text = model.items[@selected, 1]
    end
  end
  
  def mouse_down( button, origin )
    @selected = [origin.y / ($systemFont.height + 2), model.items.size - 1].min()
    
    if @selected >= 0
      @input.top = selected * ($systemFont.height + 2)
      @input.left = 125
      @input.extent = [model.columns[0][1], $systemFont.height + 2]
      @input.text = model.items[@selected, 1]
      add_child( @input )
    end
  end
  
  def draw_content()
    skip = $systemFont.height + 2

      GL::Color( 1.0, 1.0, 1.0 ); 
      GL::Begin( GL::QUADS )
        GL::Vertex(     0,      0 )
        GL::Vertex(     0, height )
        GL::Vertex( width, height )
        GL::Vertex( width,      0 )
      GL::End()
      
    if selected
      h = @selected * skip
      GL::Color( 0.8, 0.9, 1.0 ); 
      GL::Begin( GL::QUADS )
        GL::Color( 0.7, 0.85, 1.0, 0.95 )
          GL::Vertex( width, h )
          GL::Vertex(     0, h )
        GL::Color( 0.8, 0.9, 1.0, 0.95 )
          GL::Vertex(     0, h + skip )
          GL::Vertex( width, h + skip )
      GL::End()

      GL::LineWidth( 1.0 )
      half_line = 0.5
      GL::Color( 0.75, 0.75, 0.75 ); 
      GL::Begin( GL::LINE_LOOP )
        GL::Vertex(     0 - half_line,        h - half_line )
        GL::Vertex(     0 - half_line, h + half_line + skip )
        GL::Vertex( width + half_line, h + half_line + skip )
        GL::Vertex( width + half_line,        h - half_line )
      GL::End()
    end

    GL::Translate( 10, 1, 0 )
    GL::Color( 0.1, 0.1, 0.1 ); 
    
    model.items.each do | item |
      GL::PushMatrix()
        model.columns.each_with_index do | column, i |
          $systemFont.draw_string( item[i] );  GL::Translate( column[1], 0, 0 )
        end
      GL::PopMatrix()
      GL::Translate( 0, skip, 0 )
    end
  end
  
end

end
